@@ -67,6 +67,7 @@ static const unsigned char ccm_tag[] = {
 OSSL_LIB_CTX *libctx = NULL;
 const char *propq = NULL;
 
+
 int aes_ccm_encrypt(void)
 {
     int ret = 0;
@@ -95,10 +96,10 @@ int aes_ccm_encrypt(void)
 
     /* Default nonce length for AES-CCM is 7 bytes (56 bits). */
     params[0] = OSSL_PARAM_construct_size_t(OSSL_CIPHER_PARAM_AEAD_IVLEN,
-        &ccm_nonce_len);
+                                            &ccm_nonce_len);
     /* Set tag length */
     params[1] = OSSL_PARAM_construct_octet_string(OSSL_CIPHER_PARAM_AEAD_TAG,
-        NULL, ccm_tag_len);
+                                                  NULL, ccm_tag_len);
 
     /*
      * Initialise encrypt operation with the cipher & mode,
@@ -133,7 +134,7 @@ int aes_ccm_encrypt(void)
 
     /* Get tag */
     params[0] = OSSL_PARAM_construct_octet_string(OSSL_CIPHER_PARAM_AEAD_TAG,
-        outtag, ccm_tag_len);
+                                                  outtag, ccm_tag_len);
     params[1] = OSSL_PARAM_construct_end();
 
     if (!EVP_CIPHER_CTX_get_params(ctx, params))
@@ -179,11 +180,11 @@ int aes_ccm_decrypt(void)
 
     /* Set nonce length if default 96 bits is not appropriate */
     params[0] = OSSL_PARAM_construct_size_t(OSSL_CIPHER_PARAM_AEAD_IVLEN,
-        &ccm_nonce_len);
+                                            &ccm_nonce_len);
     /* Set tag length */
     params[1] = OSSL_PARAM_construct_octet_string(OSSL_CIPHER_PARAM_AEAD_TAG,
-        (unsigned char *)ccm_tag,
-        sizeof(ccm_tag));
+                                                  (unsigned char *)ccm_tag,
+                                                  sizeof(ccm_tag));
     /*
      * Initialise decrypt operation with the cipher & mode,
      * nonce length and expected tag parameters.
@@ -228,10 +229,10 @@ err:
 int main(int argc, char **argv)
 {
     if (!aes_ccm_encrypt())
-        return 1;
+        return EXIT_FAILURE;
 
     if (!aes_ccm_decrypt())
-        return 1;
+        return EXIT_FAILURE;
 
-    return 0;
+    return EXIT_SUCCESS;
 }
