@@ -83,7 +83,7 @@ static char *propq = NULL;
 
 int main(int argc, char **argv)
 {
-    int rv = EXIT_FAILURE;
+    int ret = EXIT_FAILURE;
     EVP_CIPHER *aes = NULL;
     EVP_CIPHER_CTX *aesctx = NULL;
     EVP_MAC *mac = NULL;
@@ -151,7 +151,7 @@ int main(int argc, char **argv)
      * computation below.
      */
     if (!EVP_EncryptUpdate(aesctx, composite_key + 16, &aes_len,
-            test_n, sizeof(test_n))) {
+                           test_n, sizeof(test_n))) {
         fprintf(stderr, "EVP_EncryptUpdate() failed\n");
         goto end;
     }
@@ -196,14 +196,14 @@ int main(int argc, char **argv)
         goto end;
     }
 
-    rv = EXIT_SUCCESS;
+    ret = EXIT_SUCCESS;
 end:
     EVP_CIPHER_CTX_free(aesctx);
     EVP_CIPHER_free(aes);
     EVP_MAC_CTX_free(mctx);
     EVP_MAC_free(mac);
     OSSL_LIB_CTX_free(library_context);
-    if (rv != EXIT_SUCCESS)
+    if (ret != EXIT_SUCCESS)
         ERR_print_errors_fp(stderr);
-    return rv;
+    return ret;
 }
